package data.missions.shiptestmission;

import com.fs.starfarer.api.fleet.FleetGoal;
import com.fs.starfarer.api.fleet.FleetMemberType;
import com.fs.starfarer.api.mission.FleetSide;
import com.fs.starfarer.api.mission.MissionDefinitionAPI;
import com.fs.starfarer.api.mission.MissionDefinitionPlugin;

public class MissionDefinition implements MissionDefinitionPlugin {

	public void defineMission(MissionDefinitionAPI api) {

		// Set up the fleets
		api.initFleet(FleetSide.PLAYER, "SFS", FleetGoal.ATTACK, false, 10);
		api.initFleet(FleetSide.ENEMY, "SFS", FleetGoal.ATTACK, true, 10);

		// Set a blurb for each fleet
		api.setFleetTagline(FleetSide.PLAYER, "The Sindrian Fuel Fleet.");
		api.setFleetTagline(FleetSide.ENEMY, "The Unfortunate Test Target.");

		// These show up as items in the bulleted list under
		// "Tactical Objectives" on the mission detail screen
		api.addBriefingItem("Test out the various designs the Sindrian Fuel Fleet has on offer.");
		api.addBriefingItem("Don't disappoint the Board of Research and Development.");
		api.addBriefingItem("Some of these ships may be easier to get than others.");
		api.addBriefingItem("And don't forget to sign up for the monthly company lottery!");

		// Set up the player's fleet
		api.addToFleet(FleetSide.PLAYER, "sfcsuperiapetus_Mixed", FleetMemberType.SHIP, true);
		api.addToFleet(FleetSide.PLAYER, "sfciapetus_Mixed", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.PLAYER, "sfcprometheus_barrage", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.PLAYER, "sfcepimetheus_Experimental", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.PLAYER, "sfcmenoetius_Defense", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.PLAYER, "sfcskyrend_Barrage", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.PLAYER, "sfccrius_Blaster", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.PLAYER, "sfcarke_Supporter", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.PLAYER, "sfcclepsydra_Standard", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.PLAYER, "sfcphaeton_ballistics", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.PLAYER, "sfcdrachm_Pressure", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.PLAYER, "sfcdram_assault", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.PLAYER, "sfctahlanbento_Pulser", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.PLAYER, "sfcbia_Hunter", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.PLAYER, "sfcdelos_Support", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.PLAYER, "sfchydria_Suppressor", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.PLAYER, "sfctalaria_Strike", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.PLAYER, "sfcslent_Offense", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.PLAYER, "sfcpolus_Strike", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.PLAYER, "sfccoeus_Patrol", FleetMemberType.SHIP, false);

		// Set up the enemy fleet
		api.addToFleet(FleetSide.ENEMY, "conquest_Standard", FleetMemberType.SHIP, "SFS Manager", true);

		// Set up the map.
		float width = 24000f;
		float height = 18000f;
		api.initMap((float)-width/2f, (float)width/2f, (float)-height/2f, (float)height/2f);

		float minX = -width/2;
		float minY = -height/2;

		for (int i = 0; i < 15; i++) {
			float x = (float) Math.random() * width - width/2;
			float y = (float) Math.random() * height - height/2;
			float radius = 100f + (float) Math.random() * 900f;
			api.addNebula(x, y, radius);
		}

	}

}
